package jp.sourceforge.mixedmark.itex2mmlj;

import java.io.*;

%%

%unicode
%public
%class Lexer
%implements  Parser.yyInput
/* %standalone */
%type int
%eofval{
    return YYEOF;
%eofval}

%{
static int itex2MML_lineno = 1;
static int itex2MML_rowposn = 0;
static int itex2MML_displaymode = 0;
static String itex2MML_empty_string = "";

private int token;
String yylval;
public boolean advance () throws java.io.IOException {
    token=yylex();
    return token != YYEOF;
}
public int token () {
    return token;
}
public Object value () {
    return yylval;
}

static String itex2MML_copy_string(String s) {
    return s;
}
static String itex2MML_copy_string_extra(String s,int i) {
    for (int j=0;j<i;j++)
        s = s + " ";
    return s;
}
static String itex2MML_copy2(String s,String ss) {
    return s+ss;
}
%}

%init{
//System.out.println("start!!!");
%init}

%eof{
//System.out.println("end!!!");
%eof}

%state MATH, ATTRIBUTELIST, BBOLD, FRAKTUR, CALLIGRAPHIC, ROMAN, PLAINTEXT, INT1, INT2, INT3, MATHENV, SVGENV

eol = \n|\r|\r\n

%%

<YYINITIAL>"$"{1,2}   {yybegin(MATH); itex2MML_rowposn=2; if (yytext().length()==2) { itex2MML_displaymode=1; return Parser.STARTDMATH;} itex2MML_displaymode=0; return Parser.STARTMATH;}
<YYINITIAL>"\\[" {yybegin(MATH); itex2MML_rowposn=2; itex2MML_displaymode=1; return Parser.STARTDMATH;}
<YYINITIAL>EOF    {return 0;}
<YYINITIAL>"\n" {itex2MML_lineno++; yylval=itex2MML_copy_string(yytext()); return Parser.CHAR;}
<YYINITIAL>.     {yylval=itex2MML_copy_string(yytext()); return Parser.CHAR;}


<MATH>{



"$"{1,2} |
"\\]"           {yybegin(YYINITIAL); return Parser.ENDMATH;}


"^"            {return Parser.SUP;}
"_"            {return Parser.SUB;}
"{"[\t\r\n ]*"}"   {return Parser.EMPTYMROW;}
"{"            {itex2MML_rowposn = 2; return Parser.MROWOPEN;}
"}"            {return Parser.MROWCLOSE;}

"\\left"       {itex2MML_rowposn=2; return Parser.LEFT;}
"\\right"      {return Parser.RIGHT;}

"\\big"   |
"\\bigr"  {return Parser.BIG;}
"\\Big"   |
"\\Bigr"  {return Parser.BBIG;}
"\\bigg"  |
"\\biggr" {return Parser.BIGG;}
"\\Bigg"  |
"\\Biggr" {return Parser.BBIGG;}

"\\bigl"  {return Parser.BIGL;}
"\\Bigl"  {return Parser.BBIGL;}
"\\biggl" {return Parser.BIGGL;}
"\\Biggl" {return Parser.BBIGGL;}

"\\rlap" |
"\\llap" |
"\\ulap" |
"\\dlap"               {/* Ignore */ ;}


"&amp;" |
"&"         {itex2MML_rowposn=2; return Parser.COLSEP;}

"\\&"           {yylval=itex2MML_copy_string("&amp;"); return Parser.MI;}

"\\$"           {yylval=itex2MML_copy_string("$"); return Parser.MI;}
"\\%"           {yylval=itex2MML_copy_string("%"); return Parser.MI;}

"\\\\"          {itex2MML_rowposn=2; return Parser.ROWSEP;}

[a-zA-Z]+       {yylval=itex2MML_copy_string(yytext()); return Parser.MI;}

[0-9,.]*[0-9]+          {yylval=itex2MML_copy2(yytext(), " "); return Parser.MN;}
"\\infty"   |
"\\infinity"     {yylval=itex2MML_copy_string("&infin;"); return Parser.MN;}

"("   |
"["             {yylval=itex2MML_copy_string(yytext()); return Parser.LEFTDELIM;}

")"   |
"]"             {yylval=itex2MML_copy_string(yytext()); return Parser.RIGHTDELIM;}

"\\lbrace" |
"\\{"           {yylval=itex2MML_copy_string("{"); return Parser.LEFTDELIM;}
"\\rbrace" |
"\\}"           {yylval=itex2MML_copy_string("}"); return Parser.RIGHTDELIM;}

"\\vert" |
"|"             {yylval=itex2MML_copy_string("&VerticalBar;"); return Parser.OTHERDELIM;}
"\\Vert" |
"\\|"           {yylval=itex2MML_copy_string("&DoubleVerticalBar;"); return Parser.OTHERDELIM;}

"\\setminus"   {yylval=itex2MML_copy_string("&setminus;"); return Parser.MO;}
"\\backslash"   {yylval=itex2MML_copy_string("&bsol;"); return Parser.MO;}
"\\smallsetminus"   {yylval=itex2MML_copy_string("&#xFE68;"); return Parser.MOF;}

"\\lfloor" |
"\\lceil"  |
"\\lang"   |
"\\langle"      {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.LEFTDELIM;}

"\\rfloor" |
"\\rceil"  |
"\\rang"   |
"\\rangle"      {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.RIGHTDELIM;}

"/"             {yylval=itex2MML_copy_string(yytext()); return Parser.OTHERDELIM;}

"\\uparrow"   |
"\\downarrow" |
"\\updownarrow" {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.OTHERDELIM;}

"."             {yylval=itex2MML_copy_string(yytext()); return Parser.PERIODDELIM;}

"-"    {if (itex2MML_rowposn==1) return Parser.UNARYMINUS; else {yylval=itex2MML_copy_string("&minus;"); return Parser.MO;}}

"+"    {if (itex2MML_rowposn==1) return Parser.UNARYPLUS; else {yylval=itex2MML_copy_string("+"); return Parser.MO;}}

"\\#" {yylval=itex2MML_copy_string("#"); return Parser.MOL;}

","   |
":"   |
"!"   |
"="   |
"~"   |
";"   |
"?"   |
"#"   |
"`"   |
"*"        {yylval=itex2MML_copy_string(yytext()); return Parser.MOL;}

"\\prime"  {yylval=itex2MML_copy_string("&prime;"); return Parser.MOLL;}
"'"        {yylval=itex2MML_copy_string("&prime;"); return Parser.MOL;}
"''"     {yylval=itex2MML_copy_string("&Prime;"); return Parser.MOL;}
"'''"  {yylval=itex2MML_copy_string("&#8244;"); return Parser.MOL;}
"''''"  {yylval=itex2MML_copy_string("&#8279;"); return Parser.MOL;}

"\\alpha"   |
"\\beta" |
"\\gamma" |
"\\delta" |
"\\zeta" |
"\\eta"  |
"\\theta" |
"\\iota"  |
"\\kappa" |
"\\lambda" |
"\\mu"     |
"\\nu"    |
"\\xi"   |
"\\pi"     |
"\\rho"   |
"\\sigma"  |
"\\tau"   |
"\\upsilon" |
"\\chi"  |
"\\psi"  |
"\\omega" |
"\\backepsilon"   |
"\\varepsilon" |
"\\varkappa" |
"\\varpi" |
"\\varrho" |
"\\varsigma" |
"\\vartheta"      {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.MI;}

"\\phi"  {yylval=itex2MML_copy_string("&#x03D5;"); return Parser.MI;}
"\\varphi"  {yylval=itex2MML_copy_string("&#x03C6;"); return Parser.MI;}

"\\arccos"  |
"\\arcsin"  |
"\\arctan"  |
"\\arg"     |
"\\cos"     |
"\\cosh"    |
"\\cot"     |
"\\coth"    |
"\\csc"     |
"\\deg"     |
"\\dim"     |
"\\exp"     |
"\\hom"     |
"\\ker"     |
"\\lg"      |
"\\ln"      |
"\\log"     |
"\\sec"     |
"\\sin"     |
"\\sinh"    |
"\\tan"     |
"\\tanh"    {yylval=itex2MML_copy_string(yytext()+1); return Parser.MI;}

"\\det"     |
"\\gcd"     |
"\\inf"     |
"\\lim"     |
"\\liminf"  |
"\\limsup"  |
"\\max"     |
"\\min"     |
"\\Pr"      |
"\\sup"    {yylval=itex2MML_copy_string(yytext()+1); return Parser.MIB;}

"\\omicron"  {yylval=itex2MML_copy_string("&oscr;"); return Parser.MI;}

"\\epsilon"  {yylval=itex2MML_copy_string("&epsi;"); return Parser.MI;}

"\\cdot"   {yylval=itex2MML_copy_string("&sdot;"); return Parser.MO;}

"\\Alpha"   |
"\\Beta" |
"\\Delta"  |
"\\Gamma"  |
"\\digamma"  |
"\\Lambda" |
"\\Pi"     |
"\\Phi"    |
"\\Psi"    |
"\\Sigma"  |
"\\Theta"  |
"\\Xi"     |
"\\Zeta" |
"\\Eta"  |
"\\Iota"  |
"\\Kappa" |
"\\Mu"     |
"\\Nu"    |
"\\Rho"   |
"\\Tau"  | 
"\\mho"  | 
"\\Omega"     {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.MI;}

"\\Upsilon" |
"\\Upsi" {yylval=itex2MML_copy_string("&Upsi;"); return Parser.MI;}

"\\iff" |
"\\Longleftrightarrow"  {yylval=itex2MML_copy_string("&iff;"); return Parser.MOL;}

"\\Leftrightarrow"  {yylval=itex2MML_copy_string("&Leftrightarrow;"); return Parser.MOL;}

"\\impliedby" |
"\\Leftarrow"   {yylval=itex2MML_copy_string("&Leftarrow;"); return Parser.MOL;}

"\\implies" |
"\\Rightarrow"   {yylval=itex2MML_copy_string("&Rightarrow;"); return Parser.MOL;}

"\\hookleftarrow" {yylval=itex2MML_copy_string("&hookleftarrow;"); return Parser.MOL;}

"\\embedsin" |
"\\hookrightarrow"  {yylval=itex2MML_copy_string("&hookrightarrow;"); return Parser.MOL;}

"\\longleftarrow"   {yylval=itex2MML_copy_string("&longleftarrow;"); return Parser.MOL;}

"\\longrightarrow"  {yylval=itex2MML_copy_string("&longrightarrow;"); return Parser.MOL;}

"\\leftarrow"   {yylval=itex2MML_copy_string("&leftarrow;"); return Parser.MOL;}

"\\to"    |
"\\rightarrow"  {yylval=itex2MML_copy_string("&rightarrow;"); return Parser.MOL;}

"\\leftrightarrow"  {yylval=itex2MML_copy_string("&leftrightarrow;"); return Parser.MOL;}

"\\mapsto"  |
"\\map"       {yylval=itex2MML_copy_string("&map;"); return Parser.MOL;}

"\\nearrow" |
"\\nearr"    {yylval=itex2MML_copy_string("&nearr;"); return Parser.MOL;}

"\\nwarrow" |
"\\nwarr"     {yylval=itex2MML_copy_string("&nwarr;"); return Parser.MOL;}

"\\searrow" |
"\\searr"     {yylval=itex2MML_copy_string("&searr;"); return Parser.MOL;}

"\\swarrow" |
"\\swarr"     {yylval=itex2MML_copy_string("&swarr;"); return Parser.MOL;}

"\\neArrow" |
"\\neArr"    {yylval=itex2MML_copy_string("&neArr;"); return Parser.MOL;}

"\\nwArrow" |
"\\nwArr"     {yylval=itex2MML_copy_string("&nwArr;"); return Parser.MOL;}

"\\seArrow" |
"\\seArr"     {yylval=itex2MML_copy_string("&seArr;"); return Parser.MOL;}

"\\swArrow" |
"\\swArr"     {yylval=itex2MML_copy_string("&swArr;"); return Parser.MOL;}

"\\darr"     {yylval=itex2MML_copy_string("&darr;"); return Parser.MOL;}

"\\Downarrow" {yylval=itex2MML_copy_string("&dArr;"); return Parser.MOL;}

"\\uparr"    {yylval=itex2MML_copy_string("&uarr;"); return Parser.MOL;}

"\\Uparrow" {yylval=itex2MML_copy_string("&uArr;"); return Parser.MOL;}

"\\downuparrow" |
"\\duparr" |
"\\updarr"  {yylval=itex2MML_copy_string("&updownarrow;"); return Parser.MOL;}

"\\Updownarrow"  {yylval=itex2MML_copy_string("&Updownarrow;"); return Parser.MOL;}

"\\leftsquigarrow"       {yylval=itex2MML_copy_string("&#x21DC;"); return Parser.MOL;}
"\\rightsquigarrow"      {yylval=itex2MML_copy_string("&zigrarr;"); return Parser.MOL;}
"\\leftrightsquigarrow"  {yylval=itex2MML_copy_string("&leftrightsquigarrow;"); return Parser.MOL;}
"\\upuparrows"           {yylval=itex2MML_copy_string("&upuparrows;"); return Parser.MOL;}
"\\rightleftarrows"      {yylval=itex2MML_copy_string("&rightleftarrows;"); return Parser.MOL;}
"\\rightrightarrows"     {yylval=itex2MML_copy_string("&rightrightarrows;"); return Parser.MOL;}
"\\dashleftarrow"        {yylval=itex2MML_copy_string("&#x290E;"); return Parser.MOL;}
"\\dashrightarrow"       {yylval=itex2MML_copy_string("&#x290F;"); return Parser.MOL;}
"\\curvearrowleft"       {yylval=itex2MML_copy_string("&#x293A;"); return Parser.MOL;}
"\\curvearrowbotright"   {yylval=itex2MML_copy_string("&#x293B;"); return Parser.MOL;}
"\\downdownarrows"       {yylval=itex2MML_copy_string("&downdownarrows;"); return Parser.MOL;}
"\\leftleftarrows"       {yylval=itex2MML_copy_string("&leftleftarrows;"); return Parser.MOL;}
"\\leftrightarrows"      {yylval=itex2MML_copy_string("&leftrightarrows;"); return Parser.MOL;}
"\\righttoleftarrow"     {yylval=itex2MML_copy_string("&#x27F2;"); return Parser.MOL;}
"\\lefttorightarrow"     {yylval=itex2MML_copy_string("&#x27F3;"); return Parser.MOL;}
"\\circlearrowleft"      {yylval=itex2MML_copy_string("&circlearrowleft;"); return Parser.MOL;}
"\\circlearrowright"     {yylval=itex2MML_copy_string("&circlearrowright;"); return Parser.MOL;}

"\\dots" |
"\\ldots"  {yylval=itex2MML_copy_string("&hellip;"); return Parser.MO;}
"\\cdots"   {yylval=itex2MML_copy_string("&ctdot;"); return Parser.MO;}
"\\ddots"   {yylval=itex2MML_copy_string("&dtdot;"); return Parser.MO;}
"\\udots"   {yylval=itex2MML_copy_string("&utdot;"); return Parser.MO;}
"\\vdots"   {yylval=itex2MML_copy_string("&vellip;"); return Parser.MO;}

"\\cup"  |
"\&union;" |
"\\union"          {yylval=itex2MML_copy_string("&cup;"); return Parser.MOL;}

"\\bigcup" |
"\\Union"  |
"\&Union;"      {yylval=itex2MML_copy_string("&Union;"); return Parser.MOB;}

"\\cap"  |
"\\intersection"          {yylval=itex2MML_copy_string("&cap;"); return Parser.MOL;}

"\\bigcap" |
"\\Intersection"       {yylval=itex2MML_copy_string("&Intersection;"); return Parser.MOB;}

"\\in"           {yylval=itex2MML_copy_string("&Element;"); return Parser.MOL;}

"\\gt"   |
"\\lt"   |
"\\approxeq"   |
"\\backsim"   |
"\\backsimeq"   |
"\\subset"  |
"\\subseteq" |
"\\subseteqq" |
"\\subsetneq" |
"\\subsetneqq" |
"\\varsubsetneq" |
"\\varsubsetneqq" |
"\\prec"     |
"\\parallel"  |
"\\nparallel" |
"\\shortparallel" |
"\\nshortparallel" |
"\\perp"  |
"\\eqslantgtr" |
"\\eqslantless" |
"\\gg" |
"\\ggg" |
"\\geq" |
"\\geqq" |
"\\geqslant" |
"\\gneq" |
"\\gneqq" |
"\\gnapprox" |
"\\gnsim" |
"\\gtrapprox" |
"\\ge"  |
"\\le"  |
"\\leq" |
"\\leqq" |
"\\leqslant" |
"\\lessapprox" |
"\\lessdot" |
"\\lesseqgtr" |
"\\lesseqqgtr" |
"\\lessgtr" |
"\\lneq" |
"\\lneqq" |
"\\lnsim" |
"\\lvertneqq" |
"\\gtrsim" |
"\\gtrdot" |
"\\gtreqless" |
"\\gtreqqless" |
"\\gtrless" |
"\\gvertneqq" |
"\\lesssim" |
"\\lnapprox" |
"\\nsubset" |
"\\nsubseteq" |
"\\nsubseteqq" |
"\\notin" |
"\\ni" |
"\\notni" |
"\\nmid" |
"\\nshortmid" |
"\\preceq" |
"\\npreceq" |
"\\ll" |
"\\ngeq" |
"\\ngeqq" |
"\\ngeqslant" |
"\\nleq" |
"\\nleqq" |
"\\nleqslant" |
"\\nless" |
"\\supset" |
"\\supseteq" |
"\\supseteqq" |
"\\supsetneq" |
"\\supsetneqq" |
"\\varsupsetneq" |
"\\varsupsetneqq" |
"\\approx" |
"\\asymp" |
"\\bowtie" |
"\\dashv" |
"\\Vdash" |
"\\vDash" |
"\\VDash" |
"\\vdash" |
"\\Vvdash" |
"\\models" |
"\\sim" |
"\\simeq" |
"\\nsim" |
"\\smile" |
"\\triangle" |
"\\triangledown" |
"\\triangleleft" |
"\\cong" |
"\\succ" |
"\\nsucc" |
"\\ngtr" |
"\\nsupset" |
"\\nsupseteq" |
"\\propto" |
"\\equiv" |
"\\nequiv" |
"\\frown" |
"\\triangleright" |
"\\ncong" |
"\\succeq" |
"\\succapprox" |
"\\succnapprox" |
"\\succcurlyeq" |
"\\succsim" |
"\\succnsim" |
"\\nsucceq" |
"\\nvDash" |
"\\nvdash" |
"\\nVDash" |
"\\amalg" |
"\\pm" |
"\\mp" |
"\\bigcirc" |
"\\wr" |
"\\odot" |
"\\uplus" |
"\\clubsuit" |
"\\spadesuit" |
"\\Diamond" |
"\\diamond" |
"\\sqcup" |
"\\sqcap" |
"\\sqsubset" |
"\\sqsubseteq" |
"\\sqsupset" |
"\\sqsupseteq" |
"\\Subset" |
"\\Supset" |
"\\ltimes" |
"\\div" |
"\\rtimes" |
"\\bot" |
"\\therefore" |
"\\thickapprox" |
"\\thicksim" |
"\\varpropto" |
"\\varnothing" |
"\\flat" |
"\\vee" |
"\\because" |
"\\between" |
"\\Bumpeq" |
"\\bumpeq" |
"\\circeq" |
"\\curlyeqprec" |
"\\curlyeqsucc" |
"\\doteq" |
"\\doteqdot" |
"\\eqcirc" |
"\\fallingdotseq" |
"\\multimap" |
"\\pitchfork" |
"\\precapprox" |
"\\precnapprox" |
"\\preccurlyeq" |
"\\precsim" |
"\\precnsim" |
"\\risingdotseq" |
"\\sharp" |
"\\bullet" |
"\\nexists" |
"\\dagger" |
"\\ddagger" |
"\\not" |
"\\top" |
"\\natural" |
"\\angle" |
"\\measuredangle" |
"\\backprime" |
"\\bigstar" |
"\\blacklozenge" |
"\\lozenge" |
"\\blacksquare" |
"\\blacktriangle" |
"\\blacktriangledown" |
"\\forall" |
"\\bigtriangleup" |
"\\bigtriangledown" |
"\\nprec"  {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.MOL;}

"\\aleph" |
"\\beth" |
"\\eth" |
"\\ell" |
"\\hbar" |
"\\Im" |
"\\imath" |
"\\jmath" |
"\\wp" |
"\\Re" {yylval=itex2MML_copy2(yytext(), ";"); if (yylval != itex2MML_empty_string) yylval="&"+yylval.substring(1); return Parser.MI;}

"\\Perp" |
"\\Vbar" {yylval=itex2MML_copy_string("&Vbar;"); return Parser.MOL;}
"\\Box" |
"\\square" {yylval=itex2MML_copy_string("&square;"); return Parser.MOL;}
"\\emptyset" |
"\\empty" {yylval=itex2MML_copy_string("&emptyv;"); return Parser.MI;}
"\\exists" {yylval=itex2MML_copy_string("&Exists;"); return Parser.MOL;}
"\\circ" {yylval=itex2MML_copy_string("&SmallCircle;"); return Parser.MOL;}
"\\rhd" {yylval=itex2MML_copy_string("&RightTriangle;"); return Parser.MOL;}
"\\lhd" {yylval=itex2MML_copy_string("&LeftTriangle;"); return Parser.MOL;}
"\\lll" {yylval=itex2MML_copy_string("&Ll;"); return Parser.MOL;}
"\\unrhd" {yylval=itex2MML_copy_string("&RightTriangleEqual;"); return Parser.MOL;}
"\\unlhd" {yylval=itex2MML_copy_string("&LeftTriangleEqual;"); return Parser.MOL;}
"\\Del" |
"\\nabla" {yylval=itex2MML_copy_string("&Del;"); return Parser.MOL;}
"\\sphericalangle" {yylval=itex2MML_copy_string("&angsph;"); return Parser.MOL;}

"\\heartsuit" {yylval=itex2MML_copy_string("&#x2661;"); return Parser.MOL;}
"\\diamondsuit" {yylval=itex2MML_copy_string("&#x2662;"); return Parser.MOL;}

"\\partial" {yylval=itex2MML_copy_string("&PartialD;"); return Parser.MOL;}

"\\qed" {yylval=itex2MML_copy_string("&squf;"); return Parser.MO;}

"\\mod" {yylval=itex2MML_copy_string("mod"); return Parser.MOS;}
"\\pmod" {return Parser.PMOD;}

"\\bottom" {yylval=itex2MML_copy_string("&perp;"); return Parser.MOL;}

"\\neg" {yylval=itex2MML_copy_string("&not;"); return Parser.MOL;}

"\\neq" |
"\\ne"   {yylval=itex2MML_copy_string("&ne;"); return Parser.MOL;}

"\\shortmid" |
"\\mid"  {yylval=itex2MML_copy_string("&VerticalBar;"); return Parser.MOS;}

"\\int"    |
"\\integral"     {yylval=itex2MML_copy_string("&Integral;"); return Parser.MOL;}

"\\iint"   |
"\\doubleintegral" {yylval=itex2MML_copy_string("&Int;"); return Parser.MOL;}

"\\iiint"   |
"\\tripleintegral" {yylval=itex2MML_copy_string("&tint;"); return Parser.MOL;}

"\\iiiint"   |
"\\quadrupleintegral" {yylval=itex2MML_copy_string("&qint;"); return Parser.MOL;}

"\\oint"    |
"\\conint"  |
"\\contourintegral"   {yylval=itex2MML_copy_string("&conint;"); return Parser.MOL;}

"\\times"   {yylval=itex2MML_copy_string("&times;"); return Parser.MOL;}
"\\star"   {yylval=itex2MML_copy_string("&sstarf;"); return Parser.MOL;}
"\\circleddash" |
"\\odash"   {yylval=itex2MML_copy_string("&odash;"); return Parser.MOL;}

"\\boxminus" |
"\\minusb"   {yylval=itex2MML_copy_string("&boxminus;"); return Parser.MOL;}
"\\boxplus" |
"\\plusb"   {yylval=itex2MML_copy_string("&boxplus;"); return Parser.MOL;}
"\\boxtimes" |
"\\timesb"   {yylval=itex2MML_copy_string("&boxtimes;"); return Parser.MOL;}

"\\sum"          {yylval=itex2MML_copy_string("&Sum;"); return Parser.MOB;}

"\\prod" |
"\\product"      {yylval=itex2MML_copy_string("&prod;"); return Parser.MOB;}

"\\coprod"  |
"\\coproduct"    {yylval=itex2MML_copy_string("&coprod;"); return Parser.MOB;}

"\\otimes"       {yylval=itex2MML_copy_string("&otimes;"); return Parser.MOL;}
"\\Otimes"       {yylval=itex2MML_copy_string("&Otimes;"); return Parser.MOL;}
"\\bigotimes"     {yylval=itex2MML_copy_string("&bigotimes;"); return Parser.MOL;}


"\\oplus"       {yylval=itex2MML_copy_string("&oplus;"); return Parser.MOL;}
"\\Oplus"   |
"\\bigoplus"    {yylval=itex2MML_copy_string("&bigoplus;"); return Parser.MOB;}

"\\bigodot"    {yylval=itex2MML_copy_string("&bigodot;"); return Parser.MOB;}
"\\bigsqcup"    {yylval=itex2MML_copy_string("&bigsqcup;"); return Parser.MOB;}
"\\biguplus"    {yylval=itex2MML_copy_string("&biguplus;"); return Parser.MOB;}

"\\wedge"       {yylval=itex2MML_copy_string("&wedge;"); return Parser.MOL;}
"\\Wedge"  |
"\\bigwedge"       {yylval=itex2MML_copy_string("&xwedge;"); return Parser.MOB;}

"\\Vee"  |
"\\bigvee"       {yylval=itex2MML_copy_string("&bigvee;"); return Parser.MOB;}

"\\frac"         {return Parser.FRAC;}
"\\tfrac"         {return Parser.TFRAC;}

"\\binom"        {return Parser.BINOM;}

"\\tensor"       {return Parser.TENSOR;}

"\\multiscripts" {return Parser.MULTI;}

"\\overbrace"     {return Parser.OVERBRACE;}

"\\underbrace"   {return Parser.UNDERBRACE;}

"\\bar"          {return Parser.BAR;}
"\\overline" |
"\\closure" |
"\\widebar"      {return Parser.WIDEBAR;}

"\\vec"       {return Parser.VEC;} 
"\\widevec"   {return Parser.WIDEVEC;}

"\\dot"       {return Parser.DOT;}

"\\ddot"      {return Parser.DDOT;}

"\\tilde"       {return Parser.TILDE;} 
"\\widetilde"   {return Parser.WIDETILDE;}


"\\check"       {return Parser.CHECK;} 
"\\widecheck"   {return Parser.WIDECHECK;}


"\\hat"       {return Parser.HAT;} 
"\\widehat"   {return Parser.WIDEHAT;}

"\\underset"     {return Parser.UNDER;}

"\\stackrel" |
"\\overset"      {return Parser.OVER;}

"\\over"         {return Parser.TEXOVER;}
"\\atop"         {return Parser.TEXATOP;}

"\\underoverset"  {return Parser.UNDEROVER;}

"\\sqrt"         {return Parser.SQRT;}

"\\root"         {return Parser.ROOT;}

"\\space"        {yybegin(INT1); return Parser.SPACE;}

"\\text"          {yybegin(PLAINTEXT); return Parser.TEXTBOX;}

"\\statusline"    {yybegin(PLAINTEXT); return Parser.STATLINE;}

"\\toggle"        {return Parser.TOGGLE;}

"\\fghilight" |
"\\fghighlight"   {yybegin(ATTRIBUTELIST); return Parser.FGHIGHLIGHT;}

"\\bghilight" |
"\\bghighlight"   {yybegin(ATTRIBUTELIST); return Parser.BGHIGHLIGHT;}

"\\color"        {yybegin(ATTRIBUTELIST); return Parser.COLOR;}
"\\bgcolor"        {yybegin(ATTRIBUTELIST); return Parser.BGCOLOR;}

"\\displaystyle"  {itex2MML_rowposn=2; return Parser.DISPLAY;}

"\\textstyle"     {itex2MML_rowposn=2; return Parser.TEXTSTY;}

"\\textsize"     {return Parser.TEXTSIZE;}

"\\scriptsize"     {return Parser.SCSIZE;}

"\\scriptscriptsize"     {return Parser.SCSCSIZE;}

"\\mathit"       {return Parser.ITALICS;}

"\\boldsymbol" |
"\\mathbf"       {return Parser.BOLD;}

"\\mathrm"       {yybegin(ROMAN); return Parser.RM;}

"\\mathbb"       {yybegin(BBOLD); return Parser.BB;}

"\\mathfrak" |
"\\mathfr"       {yybegin(FRAKTUR); return Parser.FRAK;}

"\\slash"       {return Parser.SLASHED;}

"\\mathcal"       {yybegin(CALLIGRAPHIC); return Parser.CAL;}

"\\begin"        {yybegin(MATHENV); return Parser.BEGINENV;}
"\\end"        {yybegin(MATHENV); return Parser.ENDENV;}

"\\substack"     {return Parser.SUBSTACK;}

"\\array"        {return Parser.ARRAY;}

"\\arrayopts"    {return Parser.ARRAYOPTS;}

"\\colalign"     {yybegin(ATTRIBUTELIST); return Parser.COLALIGN;}

"\\collayout"   {yybegin(ATTRIBUTELIST); return Parser.COLLAYOUT;}

"\\rowalign"     {yybegin(ATTRIBUTELIST); return Parser.ROWALIGN;}

"\\align"     {yybegin(ATTRIBUTELIST); return Parser.ALIGN;}

"\\equalrows"     {yybegin(ATTRIBUTELIST); return Parser.EQROWS;}

"\\equalcols"     {yybegin(ATTRIBUTELIST); return Parser.EQCOLS;}

"\\rowlines"     {yybegin(ATTRIBUTELIST); return Parser.ROWLINES;}

"\\collines"     {yybegin(ATTRIBUTELIST); return Parser.COLLINES;}

"\\frame"     {yybegin(ATTRIBUTELIST); return Parser.FRAME;}

"\\padding"     {yybegin(ATTRIBUTELIST); return Parser.PADDING;}

"\\rowopts"    {return Parser.ROWOPTS;}

"\\cellopts"   {return Parser.CELLOPTS;} 

"\\rowspan"     {yybegin(ATTRIBUTELIST); return Parser.ROWSPAN;}

"\\colspan"     {yybegin(ATTRIBUTELIST); return Parser.COLSPAN;}


"\\," |
"\\thinspace"   {return Parser.THINSPACE;}

"\\:" |
"\\medspace"    {return Parser.MEDSPACE;}

"\\;" |
"\\thickspace"  {return Parser.THICKSPACE;}

"\\quad"        {return Parser.QUAD;}
"\\qquad"       {return Parser.QQUAD;}

"\\!" |
"\\negspace"    {return Parser.NEGSPACE;}

"\\phantom"     {return Parser.PHANTOM;}

"\\href"        {yybegin(PLAINTEXT); return Parser.HREF;}

"\\operatorname" |
"\\mathop"       {yybegin(PLAINTEXT); return Parser.MATHOP;}

"\\includegraphics"(\[[^\]]*\])?\{[^}]*\}	{;}

"\\"[a-zA-Z]+    {yylval=itex2MML_copy_string(yytext()+1); return Parser.MOP;}

"\&"[a-zA-Z_][a-zA-Z0-9_]*";" |
"\&#"[0-9]+";" |
"\&#x"[a-fA-F0-9]+";"  {yylval=itex2MML_copy_string(yytext()); return Parser.MI;}

"\n"             {itex2MML_lineno++;}
[\t ]+           {;}


.              {return Parser.UNKNOWNCHAR;}
}

<ATTRIBUTELIST>"{"[ \t\n\r #a-zA-Z0-9.\-]*"}"  {
    yybegin(MATH);
    yylval=itex2MML_copy_string(yytext());
    if (yylval != itex2MML_empty_string) {
        yylval="\""+yylval.substring(1);
        yylval=yylval.substring(0,yylval.length()-1)+"\"";
    }
    return Parser.ATTRLIST;
}

<MATHENV>{
"\{matrix\}" {yybegin(MATH); return Parser.MATRIX;}
"\{pmatrix\}" {yybegin(MATH); return Parser.PMATRIX;}
"\{bmatrix\}" {yybegin(MATH); return Parser.BMATRIX;}
"\{Bmatrix\}" {yybegin(MATH); return Parser.BBMATRIX;}
"\{vmatrix\}" {yybegin(MATH); return Parser.VMATRIX;}
"\{Vmatrix\}" {yybegin(MATH); return Parser.VVMATRIX;}
"\{smallmatrix\}" {yybegin(MATH); return Parser.SMALLMATRIX;}
"\{cases\}" {yybegin(MATH); return Parser.CASES;}
"\{aligned\}" {yybegin(MATH); return Parser.ALIGNED;}
"\{gathered\}" {yybegin(MATH); return Parser.GATHERED;}
"\{split\}" {yybegin(MATH); return Parser.ALIGNED;}
"\{svg\}" {yybegin(SVGENV); return Parser.SVG;}
}

<SVGENV>{
"\\end\{svg\}" {yybegin(MATH); return Parser.ENDSVG;}
[^\\]* {yylval=itex2MML_copy_string(yytext()); yybegin(SVGENV); return Parser.XMLSTRING;}
}

<ROMAN>{
"\{"  {return Parser.ST;}
"\}"  {yybegin(MATH); return Parser.END;}
[a-zA-Z]   {yylval=itex2MML_copy_string(yytext()); return Parser.RMCHAR;}
[\t\n\r ]+       ;
.           {return ((int)(yytext().toCharArray()[0]));}
}

<BBOLD>{
"\{"  {return Parser.ST;}
"\}"  {yybegin(MATH); return Parser.END;}
[a-z] {yylval=itex2MML_copy_string(yytext()); return Parser.BBLOWERCHAR;}
[A-Z]   {yylval=itex2MML_copy_string(yytext()); return Parser.BBUPPERCHAR;}
[\t\n\r ]+       ;
.           {return ((int)(yytext().toCharArray()[0]));}
}

<CALLIGRAPHIC>{
"\{"  {return Parser.ST;}
"\}"  {yybegin(MATH); return Parser.END;}
[a-zA-Z]   {yylval=itex2MML_copy_string(yytext()); return Parser.CALCHAR;}
[\t\n\r ]+       ;
.           {return ((int)(yytext().toCharArray()[0]));}
}

<FRAKTUR>{
"\{"  {return Parser.ST;}
"\}"  {yybegin(MATH); return Parser.END;}
[a-zA-Z]   {yylval=itex2MML_copy_string(yytext()); return Parser.FRAKCHAR;}
[\t\n\r ]+       ;
.           {return ((int)(yytext().toCharArray()[0]));}
}

<PLAINTEXT>{
\{[^\}]*\}  {yylval=itex2MML_copy_string(yytext()+1); if ((yytext().length() > 1) && (yylval != itex2MML_empty_string)) yylval=yylval.substring(0,yylval.length()-2); yybegin(MATH); return Parser.TEXTSTRING;}
}

<INT1>{ 
"\{"      {return Parser.ST;}
"\}"      {yybegin(INT2); return Parser.END;}
[0-9]+    {
    yylval=itex2MML_copy_string_extra(yytext(), 1);
    if (yytext().length()>=0 && (yylval != itex2MML_empty_string)) {
        yylval=yylval.substring(0,yylval.length()-2)+".";
        yylval=yylval+yytext().substring(yytext().length()-1);
    }
    return Parser.INTONE;
}
[\t\n\r ]+   ;
.         {return ((int)(yytext().toCharArray()[0]));}
}

<INT2>{
"\{"      {return Parser.ST;}
"\}"      {yybegin(INT3); return Parser.END;}
[0-9]+    {
    yylval=itex2MML_copy_string_extra(yytext(), 1);
    if (yytext().length()>=0 && (yylval != itex2MML_empty_string)) {
        yylval=yylval.substring(0,yylval.length()-2)+".";
        yylval=yylval+yytext().substring(yytext().length()-1);
    }
    return Parser.INTTWO;
}
[\t\n\r ]+   ;
.         {return ((int)(yytext().toCharArray()[0]));}
}

<INT3>{
"\{"      {return Parser.ST;}
"\}"      {yybegin(MATH); return Parser.END;}
[0-9]+    {
    yylval=itex2MML_copy_string_extra(yytext(), 1);
    if (yytext().length()>=0 && (yylval != itex2MML_empty_string)) {
        yylval=yylval.substring(0,yylval.length()-2)+".";
        yylval=yylval+yytext().substring(yytext().length()-1);
    }
    return Parser.INTTHREE;
}
[\t\n\r ]+   ;
.         {return ((int)(yytext().toCharArray()[0]));}
}



